using System;
using System.IO;
using System.Text;
using System.Xml;

/*
Copyright (c) 2003 Roland Weigelt. All rights reserved.

This software is provided 'as-is', without any express or implied warranty. 
In no event will the authors be held liable for any damages arising from the 
use of this software.

Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim 
that you wrote the original software. If you use this software in a product, 
an acknowledgment in the product documentation would be appreciated but
is not required.

2. Altered source versions must be plainly marked as such, and must not be 
misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/

namespace ExtractXml
{
	class TheApplication
	{
		private const string _SpacesReplacingTabs="    "; // change this if you don't want 4 spaces for 1 tab
		private const string _SpecialComment="////"; // change this if you prefer a different special comment

		/// <summary>
		/// Gets the XML text from the input file using the special comments.
		/// </summary>
		/// <param name="inputFilename"></param>
		/// <returns></returns>
		private static string GetXmlText(string inputFilename)
		{
			StreamReader objInput=new StreamReader(inputFilename);
			StringBuilder stbXmlText=new StringBuilder();
			string strCurrentLine;
			string strActualLineText;
			int nTextStart;
			int nIndentationOffset=0;

			while ((strCurrentLine=objInput.ReadLine())!=null)
			{
				strCurrentLine=strCurrentLine.Replace("\t",_SpacesReplacingTabs);
				strActualLineText=strCurrentLine.TrimStart(null);

				if (strActualLineText.StartsWith(_SpecialComment))
				{
					nTextStart=_SpecialComment.Length;
					if ((strActualLineText.Length>nTextStart) && (strActualLineText[nTextStart]==' '))
						++nTextStart;

					stbXmlText.Append(strActualLineText.Substring(nTextStart));
					nIndentationOffset=strCurrentLine.Length-strActualLineText.Length;
				}
				else
				{
					if ((nIndentationOffset>0) && (strCurrentLine.Length>nIndentationOffset))
						stbXmlText.Append(strCurrentLine.Substring(nIndentationOffset));
					else
						stbXmlText.Append(strCurrentLine);
				}
				stbXmlText.Append(Environment.NewLine);
			}
			return stbXmlText.ToString();
		}

		/// <summary>
		/// Get a cleaned-up XML document from the specified XML text
		/// </summary>
		/// <param name="xmlText"></param>
		/// <returns></returns>
		private static XmlDocument GetXmlDocument(string xmlText)
		{
			// Load into DOM
			XmlDocument objXmlDocument=new XmlDocument();
			objXmlDocument.PreserveWhitespace=true;
			objXmlDocument.LoadXml(xmlText);

			// Remove all comments
			XmlNodeList objComments=objXmlDocument.DocumentElement.SelectNodes("//comment()");
			foreach (XmlNode objComment in objComments)
				objComment.ParentNode.RemoveChild(objComment);

			return objXmlDocument;
		}

		/// <summary>
		/// Main entry poing of the application
		/// </summary>
		/// <param name="args">
		/// Command line arguments, arg[0] = input, arg[1] = output
		/// </param>
		[STAThread]
		static void Main(string[] args)
		{
			int nExitCode=0;
			if (args.Length!=2)
			{
				Console.Error.WriteLine("Usage: ExtractXml inputFile.cs outputFile.xml");
				Environment.Exit(1);
			}

			string strInputFile=null;
			string strOutputFile=null;
			try
			{
				strInputFile=Path.GetFullPath(args[0]);
				strOutputFile=Path.GetFullPath(args[1]);
			}
			catch (Exception exc)
			{
				Console.Error.Write("ExtractXml: ");
				if (strInputFile==null)
					Console.Error.WriteLine(String.Format("Error in first parameter ({0}):",args[0]));
				else
					Console.Error.WriteLine(String.Format("Error in second parameter ({0}):",args[1]));
				Console.Error.WriteLine(exc.Message);
				Environment.Exit(1);
			}
			try
			{
				string strXmlText=GetXmlText(Path.GetFullPath(strInputFile));
				XmlDocument objOutputXml=GetXmlDocument(strXmlText);
				objOutputXml.Save(Path.GetFullPath(strOutputFile));
			}
			catch (Exception exc)
			{
				Console.Error.WriteLine(
					String.Format("ExtractXml\nInput file: {0}\nOutput file: {1}\nReported error: {2}",strInputFile,strOutputFile,exc.Message));
				nExitCode=1;
			}
			Environment.Exit(nExitCode);
		}
	}
}
